!------------------------------------------------------------------------------------------------------------
! Library Name ....  TPicture
! Description .....  Taboga Field Pictures Library
! Version .........  1.1
! Copyright  2005-2009 by Edgard L. Riba (Taboga Software - www.tabogasoftware.com)
! All rights reserved
!
! Revision History
!   May 2009         DeformatNumber was added for support of report outputs....
!   Mar 2009         Library ver 1.1 was completed (Clarion 7)
!   Apr 2005         Library ver 1.0 was completed
!------------------------------------------------------------------------------------------------------------

  MEMBER()

  INCLUDE('TPicture.INC'),ONCE
    map
__IsDigit                         PROCEDURE(STRING pTestValue),BYTE
__replaceChar                     PROCEDURE(*CSTRING pStr,STRING pWhat,STRING pWith)
    END
          

!
!::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
! Init
!::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!------------------------------------------------------------------------------------------------------------
TPictureClass.CONSTRUCT        PROCEDURE
!------------------------------------------------------------------------------------------------------------
        CODE
   SELF.Inited = TRUE
   SELF.strictDeformat = TRUE           !Should we be strict about errors during deformat?
   RETURN 

!------------------------------------------------------------------------------------------------------------
TPictureClass.Destruct         PROCEDURE
!------------------------------------------------------------------------------------------------------------
  CODE
  IF SELF.Inited
     SELF.Inited = FALSE
  END
  RETURN

!
!::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
! Field handling
!::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!------------------------------------------------------------------------------------------------------------
TPictureClass.TakePicture               PROCEDURE(STRING pPicture)
!------------------------------------------------------------------------------------------------------------
p               CSTRING(101)
i               SHORT(0)
j               SHORT(0)
tmp             CSTRING(101)
xIndex          LONG
xLeadingZero    BYTE(0)
xLeadingAsterik BYTE(0)
xAllowSeparator BYTE(1)
xSeparator      STRING(1)
xExcelSeparator STRING(1)
xLen            LONG(0)
!
!Numeric
xReadingCurrency     BYTE(0)
xReadingSize         BYTE(0)
xReadingDecimals     BYTE(0)
xSize                CSTRING(101)
xDigits              BYTE               !Number of digits...
xDigitMask           CSTRING(101)
xDecimals            CSTRING(101)
xDecimalDigits       BYTE               !Number of decimal digits...
xDecimalMask         CSTRING(101)   
xCurrencySymbol      CSTRING(51)
eDONE                EQUATE(2)          !To signal when finished reading a section!

        CODE
   SELF.screenMask     = ''
   SELF.screenMaskLen  = 0
   SELF.excelMask      = ''
   !
   !
   SELF.currencySymbol = ''
   SELF.currencyPos    = ''
   SELF.decimalSeparator = ''                   
   SELF.groupCharacter   = ','
   SELF.negativeSign     = 'NONE'
   SELF.allowBlank       = FALSE
   SELF.leadingCharacter = 'CLIP'
   xSize     = ''
   xDecimals = ''
   !     
   !Special case...
   IF pPicture='' OR pPicture = '@'
      DO BadPicture
   END
   !
   !Clean up the picture
   SELF.picture = CLIP(pPicture)
   p = SELF.picture
   IF p[1] = '@'
      p = SUB(p,2,LEN(p)-1)
   END
   SELF.pictureType = UPPER(p[1])               !Store the picture type for latter use...
   !
   !
   CASE UPPER(p[1])
   !::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
   OF 'S'               !STRING
   !::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
      xLen = SUB(p,2,LEN(p)-1)
      IF xLen           !Valid
         SELF.screenMaskLen = xLen
         SELF.screenMask    = ALL('$',xLen)               !$$$$$$$$ for @S8
         SELF.ExcelMask     = 'General'
         DO GoodPicture
      ELSE   
         DO BadPicture
      END   
   !::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
   OF 'P'               !PATTERN AND KEY-IN
   OROF 'K'
   !::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
      j = LEN(p)
      IF j > 2          !i.e. it is not @PP or @KK
         SELF.screenMask = p[2 : j-1]
         SELF.ExcelMask  = 'General'
         DO GoodPicture
      ELSE
         DO BadPicture
      END
   !::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
   OF 'E'               !SCIENTIFIC
   !::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
      j = LEN(p)
      SELF.negativeSign = 'TRAILING'                    !Scientific are always with negative trailing sign...
      !
      !FIRST we need to PARSE the picture
      j = LEN(p)
      LOOP i=2 TO j
         IF xReadingSize = TRUE                                 !Are we reading the size of the numeric picture?
            IF p[i] = '.' 
               IF p[i+1] = '.'                          !20..2
                  SELF.groupCharacter   = '.'
                  SELF.decimalSeparator = '.'
                  i += 1
                  xReadingSize = eDONE
               ELSIF p[i+1] = '`'                       !20.`2
                  SELF.groupCharacter   = '.'
                  SELF.decimalSeparator = ','
                  i += 1
                  xReadingSize = eDONE
               ELSE                                     !20.2
                  SELF.groupCharacter   = ','
                  SELF.decimalSeparator = '.'
                  xReadingSize = eDONE
               END
            ELSIF p[i] = '`'                            
               IF p[i+1] = '.'                          !20`.2
                  SELF.groupCharacter   = ','
                  SELF.decimalSeparator = '.'
                  xReadingSize = eDONE
               ELSIF p[i+1] = '`'                       !20``2
                  SELF.groupCharacter   = ','
                  SELF.decimalSeparator = ','
                  xReadingSize = eDONE
               ELSE                                     !20`2       
                  SELF.groupCharacter   = '.'
                  SELF.decimalSeparator = ','
                  xReadingSize = eDONE
               END   
            ELSIF p[i] = '_'                            
               SELF.groupCharacter      = '<60>'
               IF p[i+1] = '.'                          !20_.2
                  SELF.decimalSeparator = '.'
                  i += 1
                  xReadingSize = eDONE
               ELSIF p[i+1] = '`'                       !20_`2
                  SELF.decimalSeparator = ','
                  i += 1
                  xReadingSize = eDONE
               END
            ELSIF __IsDigit(p[i])                         !a numeric digit, part of the picture size   
               xSize = xSize & p[i]
            ELSE 
               DO BadPicture
            END   
            CYCLE
         END
         IF xReadingDecimals                            !If we are reading decimals, and find
            IF __IsDigit(p[i])                          !..a numeric digit
               xDecimals = xDecimals & p[i]
               CYCLE
            ELSE
               xReadingDecimals = eDONE                 !..signal that we are done with the decimals, and continue...
            END   
         END
         IF __IsDigit(p[i])                             !We found either either the 
            IF NOT xReadingSize                         !..start of the picture size
               xSize = p[i]
               xReadingSize = TRUE
               CYCLE
            ELSIF xReadingSize = eDONE AND NOT xReadingDecimals  !..or the start of the picture decimals...
               xDecimals = p[i]
               xReadingDecimals = TRUE
               CYCLE
            END
         END
         IF p[i] = 'B'                                  !We are done with the negative sign
            SELF.AllowBlank = TRUE
            CYCLE
         END   
      END
      !
      !SECOND we build the mask...
      SELF.ScreenMask = '-'
      xDecimalMask = ''
      xDigitMask   = ''
      xDigits      = xDecimals                          !Actually not decimals, but digits before decimal point...        
      IF xDigits
         j = 0
         LOOP i=1 TO xDigits
            IF j = 3 
               xDigitMask = '#' & SELF.GroupCharacter & xDigitMask
               j = 1
            ELSE
               xDigitMask = '#' & xDigitMask
               j += 1 
            END
         END
         xDecimalDigits = xSize - 6 - LEN(xDigitMask) - 1
         IF xDecimalDigits
            xDecimalMask   = SELF.decimalSeparator & ALL('#',xDecimalDigits)
         END
      ELSE
         xDigits      = xSize - 6
         xDigitMask   = ALL('#',xDigits)
         xDecimalMask = ''
      END
      SELF.ScreenMask = SELF.ScreenMask & xDigitMask & xDecimalMask & 'E###'
      !
      !THIRD - EXCEL...
      SELF.ExcelMask = '##0.0E+0'
      DO GoodPicture
   !::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
   OF 'N'               !NUMERIC AND CURRENCY...
   !::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
      !
      !FIRST we need to PARSE the picture
      j = LEN(p)
      LOOP i=2 TO j
         IF xReadingCurrency = TRUE                             !Are we reading the currency symbol?
            IF p[i] = '~'                                       !...here we finish
               xReadingCurrency = eDONE
            ELSE
               SELF.currencySymbol = SELF.currencySymbol & p[i]
            END   
            CYCLE
         END   
         IF p[i] = '$' OR p[i] = '~'                            !Currency symbol starts, or a dollar symbol...
            IF NOT xSize                                        
               SELF.currencyPos = 'LEADING'
            ELSE
               SELF.currencyPos = 'TRAILING'
            END   
            IF p[i] = '~'
               xReadingCurrency = TRUE                          !..signal we are reading the currency symbols...
            ELSE
               SELF.currencySymbol = '$'
            END   
            CYCLE
         END   
         IF xReadingSize = TRUE                                 !Are we already reading the size of the numeric picture?
            IF p[i] = '.'                                       !
               IF i=j                                           !20 = No decimal points...
                  SELF.groupCharacter   = ','
                  xReadingSize = eDone
               ELSIF p[i+1] = '.'                               !20..2  
                  SELF.groupCharacter   = '.'
                  SELF.decimalSeparator = '.'
                  i += 1
                  xReadingSize = eDONE
               ELSIF p[i+1] = '`'                               !20.`2   
                  SELF.groupCharacter   = '.'
                  SELF.decimalSeparator = ','
                  i += 1
                  xReadingSize = eDONE
               ELSIF p[i+1] = 'v'                               !20.v2
                  SELF.groupCharacter   = '.'
                  SELF.decimalSeparator = '#'
                  i += 1
                  xReadingSize = eDONE
               ELSE                                             !20.2
                  SELF.groupCharacter   = ','
                  SELF.decimalSeparator = '.'
                  xReadingSize = eDONE
               END
               CYCLE
            ELSIF p[i] = '`'                                    !20`2           (20``2 doesn't exist)...
               SELF.groupCharacter   = '.'                      !
               SELF.decimalSeparator = ','
               xReadingSize = eDONE
               CYCLE
            ELSIF p[i] = 'v'                                    !20v2           (20`v2 doesn't exist)...
               SELF.groupCharacter   = ','
               SELF.decimalSeparator = '#'
               xReadingSize = eDONE
               CYCLE
            ELSIF p[i] = '_'                            
               SELF.groupCharacter      = '<60>'
               IF i=j                                           !20_
                  xReadingSize = eDONE
               ELSIF p[i+1] = '.'
                  SELF.decimalSeparator = '.'                   !20_.2
                  i += 1
                  xReadingSize = eDONE
               ELSIF p[i+1] = '`'                               !20_`2
                  SELF.decimalSeparator = ','
                  i += 1
                  xReadingSize = eDONE
               ELSIF p[i+1] = 'v'                               !20_v2
                  SELF.decimalSeparator = '#'
                  i += 1
                  xReadingSize = eDONE
               END
               CYCLE
            ELSIF p[i] = '-'
               IF i=j                                           !20-                    !Trailing negative....
                  xReadingSize = eDONE
                  SELF.negativeSign = 'TRAILING'
               ELSE   
                  SELF.groupCharacter      = '-'
                  IF p[i+1] = '.'                               !20-.2
                     SELF.decimalSeparator = '.'
                     i += 1
                     xReadingSize = eDONE
                  ELSIF p[i+1] = '`'                            !20-`2
                     SELF.decimalSeparator = ','
                     i += 1
                     xReadingSize = eDONE
                  ELSIF p[i+1] = 'v'                            !20-v2
                     SELF.decimalSeparator = '#'
                     i += 1
                     xReadingSize = eDONE
                  ELSIF p[i+1] = '-'                            !20--
                     SELF.decimalSeparator = ''                 !.....No decimal
                     SELF.negativeSign = 'TRAILING'
                     i += 1
                     xReadingSize = eDONE
                  END
               END   
               CYCLE
            ELSIF __IsDigit(p[i])                       !a numeric digit, part of the picture size   
               xSize = xSize & p[i]
               CYCLE
            END   
         END
         IF xReadingDecimals                            !If we are reading decimals, and find
            IF __IsDigit(p[i])                          !..a numeric digit
               xDecimals = xDecimals & p[i]
               CYCLE
            ELSE
               xReadingDecimals = eDONE
            END   
         END
         IF __IsDigit(p[i])                             !We found a digit
            IF NOT xReadingSize                         !..start of the picture size,
               IF p[i] = 0
                  SELF.leadingCharacter = 'ZERO'
               ELSE
                  xSize = p[i]
               END   
               xReadingSize = TRUE
               CYCLE
            ELSIF xReadingSize = eDONE AND NOT xReadingDecimals  !..or the start of the picture decimals...
               xDecimals = p[i]
               xReadingDecimals = TRUE
               CYCLE
            END
         END
         IF p[i] = '-'                          !Negative sign
            IF NOT xReadingSize
               SELF.negativeSign = 'LEADING'
            ELSE    
               SELF.negativeSign = 'TRAILING'
            END    
            CYCLE
         END   
         IF p[i] = '('                          !Negative sign
            ASSERT(NOT xReadingSize)
            SELF.negativeSign = 'BRACKETS'
            CYCLE
         END   
         IF p[i] = ')'                          !We are done with the negative sign
            ASSERT(SELF.negativeSign = 'BRACKETS')
            CYCLE
         END   
         IF p[i] = 'B'                          !We are done with the negative sign
            SELF.AllowBlank = TRUE
            CYCLE
         END   
         IF p[i] = '_'                          
            SELF.leadingCharacter = 'SPACE'
            CYCLE
         END   
         IF p[i] = '*'                          
            SELF.leadingCharacter = 'ASTERISK'
            CYCLE
         END   
      END
      SELF.significantDigits = xSize
      SELF.DecimalDigits     = xDecimals
      !
      !SECOND - we have parsed the numeric picture and can now BUILD the mask...
      xDecimalDigits = xDecimals
      IF SELF.currencyPos  = 'LEADING'
         xCurrencySymbol = SELF.currencySymbol
         __replaceChar(xCurrencySymbol,' ','<60>')
         SELF.ScreenMask = xCurrencySymbol
      END   
      IF SELF.negativeSign = 'LEADING'
         SELF.ScreenMask = SELF.ScreenMask & '-'
      ELSIF SELF.negativeSign = 'BRACKETS'   
         SELF.ScreenMask = SELF.ScreenMask & '('
      END   
      xDigits = xSize - CHOOSE(SELF.negativeSign='BRACKETS',2,CHOOSE(SELF.negativeSign='NONE',0,1)) - LEN(SELF.currencySymbol) - xDecimalDigits 
      IF xDecimalDigits                             !If the picture has decimals
         xDigits = xDigits - 1
      END
      IF xDigits < 0 
         DO BadPicture
      END   
      IF xDecimalDigits                             !If the picture has decimals
         xDecimalMask = SELF.decimalSeparator & ALL('#',xDecimalDigits)
      ELSE   
         xDecimalMask = ''
      END
      IF xDigits
         CASE SELF.leadingCharacter 
         OF 'CLIP'                                      !Default...
            xDigitMask = '#'
            j          = 1
            LOOP i=1 TO xDigits-1
              IF j = 3 AND i < xDigits-1
                 xDigitMask = SELF.GroupCharacter & xDigitMask
                 j = 0
              ELSE
                 xDigitMask = '<60>' & xDigitMask
                 j += 1 
              END
            END   
         OF 'ZERO'                                      !As in @n011        
            xDigitMask = ALL('#',xDigits)
         OF 'SPACE'                                     !As in @n_11
            IF xDigits = 1
               xDigitMask = '#'
            ELSE
               xDigitMask = ALL('<60>',xDigits-1) & '#'
            END   
         OF 'ASTERISK'                                  !As in @n*11
            IF xDigits = 1
               xDigitMask = '#'
            ELSE
               xDigitMask = ALL('*',xDigits-1) & '#'
            END   
         END
      END
      SELF.ScreenMask = SELF.ScreenMask & xDigitMask & xDecimalMask
      IF SELF.currencyPos  = 'TRAILING'
         xCurrencySymbol = SELF.currencySymbol
         __replaceChar(xCurrencySymbol,' ','<60>')
         SELF.ScreenMask = SELF.ScreenMask & xCurrencySymbol
      END   
      IF SELF.negativeSign = 'TRAILING'
         SELF.ScreenMask = SELF.ScreenMask & '-'
      ELSIF SELF.negativeSign = 'BRACKETS'   
         SELF.ScreenMask = SELF.ScreenMask & ')'
      END   
      !
      !THIRD - we build the excel mask......
      !
      !If no decimal digits, and no thousand separator (e.g. SPACE leading character or @n4), and no currencysymbol, and 
      !...leading negative sign or no negative sign ---> "General"
      !...e.g. @n_11
      IF SELF.DecimalDigits = 0 AND (SELF.leadingCharacter = 'SPACE' OR (SELF.leadingCharacter='CLIP' AND SELF.significantDigits = 4)) AND |
         (SELF.negativeSign='NONE' OR SELF.negativeSign='LEADING') AND NOT SELF.CurrencyPos
         SELF.excelMask = 'General'
      ELSE
         IF SELF.currencyPos  = 'LEADING'
            SELF.excelMask = SELF.currencySymbol
         END
         !
         xExcelSeparator = SELF.GroupCharacter
         IF SELF.GroupCharacter                    !Force COMMA
            xExcelSeparator = ','
         END
         IF xExcelSeparator = '#' 
            xExcelSeparator = ' '
         END
         !
         !
         IF SELF.leadingCharacter = 'ZERO'
            SELF.excelMask = SELF.excelMask & ALL('0',xDigits)
         ELSIF SELF.leadingCharacter = 'SPACE'   
            SELF.excelMask = SELF.excelMask & '0'
         ELSE
            SELF.excelMask = SELF.excelMask & '#' & xExcelSeparator & '##0'
         END   
         !
         xExcelSeparator = SELF.decimalSeparator
         IF SELF.decimalSeparator
            xExcelSeparator = '.'                  !Force PERIOD...
         END    
         IF xExcelSeparator = '#' 
            xExcelSeparator = ' '
         END
         SELF.excelMask = SELF.excelMask & xExcelSeparator & ALL('0',xDecimalDigits)
         !
         IF SELF.currencyPos  = 'TRAILING'
            SELF.excelMask = SELF.excelMask & SELF.currencySymbol
         END
         !
         CASE SELF.negativeSign 
         OF 'LEADING'
            SELF.excelMask = SELF.excelMask & ';-' & SELF.excelMask
         OF 'TRAILING'
            SELF.excelMask = SELF.excelMask & ';' & SELF.excelMask & '-'
         OF 'BRACKETS'
            SELF.excelMask = SELF.excelMask & ';(' &  SELF.excelMask & ')'
         END
      END   
      DO GoodPicture
   !::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
   OF 'D'               !DATES...
   !::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
      j = LEN(p)
      xSeparator       = '/'
      xExcelSeparator  = '/'
      xIndex = DEFORMAT(p)
      IF p[2] = '0'                                             !@D01
         xLeadingZero = TRUE
      ELSE                                                      !@D1
         xLeadingZero = FALSE
      END
      IF p[2] = '*'                                             !@D*1
         xLeadingAsterik = TRUE
      ELSE                                                      !@D1
         xLeadingAsterik = FALSE
      END
      CASE xIndex
      OF 1                                                      !mm/dd/yy
         IF xLeadingZero
            SELF.ScreenMask = '##=##=##'
            SELF.ExcelMask  = 'mm=dd=yy'
         ELSIF xLeadingAsterik   
            SELF.ScreenMask = '**=**=**'
            SELF.ExcelMask  = 'mm=dd=yy'
         ELSE
            SELF.ScreenMask = '<60>#=##=##'                     !...<#=##=##
            SELF.ExcelMask  = 'm=dd=yy'
         END
      OF 2                                                      !mm/dd/yyyy
         IF xLeadingZero
            SELF.ScreenMask = '##=##=####'
            SELF.ExcelMask  = 'mm=dd=yyyy'
         ELSIF xLeadingAsterik   
            SELF.ScreenMask = '**=**=****'
            SELF.ExcelMask  = 'mm=dd=yyyy'
         ELSE
            SELF.ScreenMask = '<60>#=##=####'
            SELF.ExcelMask  = 'm=dd=yyyy'
         END
      OF 3                                                      !mmm dd,yyyy
         xSeparator = ','
         IF xLeadingZero
            SELF.ScreenMask = '### ##=####'                              
            SELF.ExcelMask  = 'mmm dd=yyyy'                              
         ELSIF xLeadingAsterik   
            SELF.ScreenMask = '*** **=****'    
            SELF.ExcelMask  = 'mmm dd=yyyy'                              
         ELSE
            SELF.ScreenMask = '<60>{3} <60>#=####'                              
            SELF.ExcelMask  = 'mmm d=yyyy'                              
         END
      OF 4                                                      !mmmmmmmmmmmm dd,yyyy
         xSeparator = ','
         IF xLeadingZero
            SELF.ScreenMask = '############ ##=####'                     
            SELF.ExcelMask  = 'Mmmm dd=yyyy'                              
         ELSIF xLeadingAsterik   
            SELF.ScreenMask = '************ **=****'                     
            SELF.ExcelMask  = 'Mmmm dd=yyyy'                              
         ELSE
            SELF.ScreenMask = '<60>{12} <60>#=####'                     
            SELF.ExcelMask  = 'Mmmm d=yyyy'                              
         END
      OF 5                                                      !dd/mm/yy
         IF xLeadingZero
            SELF.ScreenMask = '##=##=##'
            SELF.ExcelMask  = 'dd=mm=yy'
         ELSIF xLeadingAsterik   
            SELF.ScreenMask = '**=**=**'
            SELF.ExcelMask  = 'dd=mm=yy'
         ELSE
            SELF.ScreenMask = '<60>#=##=##'
            SELF.ExcelMask  = 'd=mm=yy'
         END
      OF 6                                                      !dd/mm/dd
         IF xLeadingZero
            SELF.ScreenMask = '##=##=####'
            SELF.ExcelMask  = 'dd=mm=yyyy'
         ELSIF xLeadingAsterik   
            SELF.ScreenMask = '**=**=****'
            SELF.ExcelMask  = 'dd=mm=yyyy'
         ELSE
            SELF.ScreenMask = '<60>#=##=####'
            SELF.ExcelMask  = 'd=mm=yyyy'
         END
      OF 7                                                      !dd mmm yy
         xSeparator = '<60>'
         xExcelSeparator = ' '
         IF xLeadingZero
            SELF.ScreenMask = '##=###=##'
            SELF.ExcelMask  = 'dd=mmm=yy'
         ELSIF xLeadingAsterik   
            SELF.ScreenMask = '**=***=**'
            SELF.ExcelMask  = 'dd=mmm=yy'
         ELSE
            SELF.ScreenMask = '<60>#=<60,60,60>=##'             !.... <#=<<<=##    
            SELF.ExcelMask  = 'd=mmm=yy'
         END
      OF 8                                                      !dd mmm yyyy
         xSeparator = '<60>'
         xExcelSeparator = ' '
         IF xLeadingZero
            SELF.ScreenMask = '##<60>###<60>####'
            SELF.ExcelMask  = 'dd=mmm=yyyy'
         ELSIF xLeadingAsterik   
            SELF.ScreenMask = '**=***=****'
            SELF.ExcelMask  = 'dd=mmm=yyyy'
         ELSE
            SELF.ScreenMask = '<60>#<60>{5}####'                !.... <#=<<<<<=##    
            SELF.ExcelMask  = 'd=mmm=yyyy'
         END
      OF 9                                                      !yy/mm/dd
         IF xLeadingZero
            SELF.ScreenMask = '##=##=##'
         ELSIF xLeadingAsterik   
            SELF.ScreenMask = '**=**=**'
         ELSE
            SELF.ScreenMask = '##=##=##'
         END
         SELF.ExcelMask  = 'yy=mm=dd'
      OF 10                                                     !yyyy/mm/dd
         IF xLeadingZero
            SELF.ScreenMask = '####=##=##'
         ELSIF xLeadingAsterik   
            SELF.ScreenMask = '****=**=**'
         ELSE
            SELF.ScreenMask = '####=##=##'
         END
         SELF.ExcelMask  = 'yyyy=mm=dd'
      OF 11                                                     !yymmdd
         xAllowSeparator = FALSE                                !...Picture doesn't use separators...
         IF xLeadingAsterik   
            SELF.ScreenMask = '******'
         ELSE
            SELF.ScreenMask = '######'
         END
         SELF.ExcelMask  = 'yymmdd'
      OF 12                                                     !yyyymmdd
         xAllowSeparator = FALSE                                !...Picture doesn't use separators...
         IF xLeadingAsterik
            SELF.ScreenMask = '********'
         ELSE
            SELF.ScreenMask = '########'
         END
         SELF.ExcelMask  = 'yyyymmdd'
      OF 13                                                     !mm/yy
         IF xLeadingAsterik   
            SELF.ScreenMask = '**=**'
         ELSE
            SELF.ScreenMask = '##=##'
         END
         SELF.ExcelMask  = 'mm=yy'
      OF 14                                                     !mm/yyyy
         IF xLeadingAsterik   
            SELF.ScreenMask = '**=****'
         ELSE
            SELF.ScreenMask = '##=####'
         END
         SELF.ExcelMask  = 'mm=yyyy'
      OF 15                                                     !yy/mm
         IF xLeadingAsterik   
            SELF.ScreenMask = '**=**'
         ELSE
            SELF.ScreenMask = '##=##'
         END
         SELF.ExcelMask  = 'yy=mm'
      OF 16                                                     !yyyy/mm
         IF xLeadingAsterik   
            SELF.ScreenMask = '****=**'
         ELSE
            SELF.ScreenMask = '####=##'
         END
         SELF.ExcelMask  = 'yyyy=mm'
      OF 17                                                     !Windows short
         IF xLeadingZero
            SELF.ScreenMask = '##=##=####'
         ELSIF xLeadingAsterik   
            SELF.ScreenMask = '**=**=****'
         ELSE
            SELF.ScreenMask = '<60>#=##=####'
         END
         SELF.ExcelMask  = 'd=mmm=yyyy'
      OF 18                                                     !Windows Long
         xAllowSeparator = FALSE                                !Picture doesn't use separators...
         SELF.ScreenMask = '##<60>{17}####'
         SELF.ExcelMask  = 'mmmm d, yyyy'
      END
      IF xAllowSeparator
         LOOP i=2 TO j
            IF __IsDigit(p[i])                                       !0..9
               CYCLE
            ELSE                                                   !Separators 
               CASE p[i]
               OF '.'                                              !..Period
                  xSeparator = '.'
                  xExcelSeparator = '.'
               OF '-'                                              !..Dash
                  xSeparator = '-'
                  xExcelSeparator = '-'
               OF '_'                                              !..Space
                  xSeparator = '<<'
                  xExcelSeparator = ' '
               OF '`'                                              !..Comma 
                  xSeparator = ','
                  xExcelSeparator = ','
               END
            END
            BREAK
         END
         __replaceChar(SELF.screenMask,'=',xSeparator)
         __replaceChar(SELF.ExcelMask,'=',xExcelSeparator)
      END   
      DO GoodPicture
   OF 'T'               !TIME...
      xSeparator = ':'
      j = LEN(p)
      xIndex = DEFORMAT(p)
      IF p[2] = '0'                                             !@T01
         xLeadingZero = TRUE
      ELSE                                                      !@T1
         xLeadingZero = FALSE
      END
      CASE xIndex
      OF 1                                                      !hh:mm
         IF xLeadingZero
            SELF.ScreenMask = '<60>#=##'
            SELF.ExcelMask  = 'h=mm'
         ELSE
            SELF.ScreenMask = '##=##'
            SELF.ExcelMask  = 'hh=mm'
         END
      OF 2                                                      !hhmm
         xAllowSeparator = FALSE                                !...Picture doesn't use separators...
         SELF.ScreenMask = '####'
         SELF.ExcelMask  = 'hhmm'
      OF 3                                                      !hh:mmXM
         IF xLeadingZero
            SELF.ScreenMask = '<60>#=##XM'
            SELF.ExcelMask  = 'h=mm AM/PM'
         ELSE
            SELF.ScreenMask = '##=##XM'
            SELF.ExcelMask  = 'hh=mm AM/PM'
         END
      OF 4                                                      !hh:mm:ss
         IF xLeadingZero
            SELF.ScreenMask = '<60>#=##=##'
            SELF.ExcelMask  = 'h=mm=ss'
         ELSE
            SELF.ScreenMask = '##=##=##'
            SELF.ExcelMask  = 'hh=mm=ss'
         END
      OF 5                                                      !hhmmss
         xAllowSeparator = FALSE                                !...Picture doesn't use separators...
         SELF.ScreenMask = '######'
         SELF.ExcelMask  = 'hhmmss'
      OF 6                                                      !hh:mm:ssXM
         IF xLeadingZero
            SELF.ScreenMask = '<60>#=##=##XM'
            SELF.ExcelMask  = 'h=mm=ss AM/PM'
         ELSE
            SELF.ScreenMask = '##=##=##XM'
            SELF.ExcelMask  = 'hh=mm=ss AM/PM'
         END
      OF 7                                                      !Window Short
         SELF.ScreenMask = '##=## XM'
         SELF.ExcelMask  = 'hh=mm AM/PM'
      OF 8                                                      !Windows Long
         SELF.ScreenMask = '##=##=## XM'
         SELF.ExcelMask  = 'h=mm=ss AM/PM'
      END   
      IF xAllowSeparator
         LOOP i=2 TO j
            IF __IsDigit(p[i])                                       !0..9
               CYCLE
            ELSE                                                   !Separators 
               CASE p[i]
               OF '.'                                              !..Period
                  xSeparator = '.'
                  xExcelSeparator = '.'
               OF '-'                                              !..Dash
                  xSeparator = '-'
                  xExcelSeparator = '-'
               OF '_'                                              !..Space
                  xSeparator = '<60>'
                  xExcelSeparator = ' '
               OF '`'                                              !..Comma
                  xSeparator = ','
                  xExcelSeparator = ','
               END
            END
            BREAK
         END
         __replaceChar(SELF.ScreenMask,'=',xSeparator)
         __replaceChar(SELF.ExcelMask,'=',xExcelSeparator)
      END   
      DO GoodPicture
   END
   
BadPicture              ROUTINE   
   SELF.picture       = ''
   SELF.screenMask    = ''
   SELF.screenMaskLen = 0
   SELF.ExcelMask     = 0
   RETURN 0

GoodPicture             ROUTINE   
   SELF.screenMaskLen = LEN(SELF.screenMask)
   RETURN SELF.screenMaskLen


!------------------------------------------------------------------------------------------------------------
TPictureClass.DeformatNumber                PROCEDURE(STRING pValue)
!Given a value in a string, this procedure deformats it according to its' picture...It works for NUMERIC,
!..and SCIENTIFIC pictures...
!------------------------------------------------------------------------------------------------------------
xResult         LONG
xStr            &CSTRING
xInitPos        BYTE(1)
xLen            LONG,AUTO
xPos            LONG,AUTO
xValue          REAL
xIsNegativeValue BYTE(0)
xIsBadPicture   BYTE(0)
xDecimalsRead   BYTE(0)
xDecimalPart    CSTRING(51)
xIntegerPart    CSTRING(51)
xIntegersRead   BYTE(0)
xCurrencySymbol CSTRING(21)
xExp            SHORT
xExpS           STRING(4)
        CODE
   ASSERT(SELF.Picture <> '','Programming error on call to TPictureClass.Deformat.   Picture has not yet been set...')
   IF pValue = ''
      xValue = 0
      DO ProcedureReturn
   END
   CASE SELF.pictureType
   OF 'N'
      xLen = LEN(CLIP(LEFT(pValue)))    !Size of trimmed string to deformat...
      xStr &= NEW(CSTRING(xLen + 1))    !Make a local CSTRING to hold it,
      xStr = CLIP(LEFT(pValue))         !RIGHT(pValue,xLen)         !..and copy it...
      !
      !
      !(1)  Check for leading currency....If found we remove it from what we need to evaluate....
      IF SELF.currencyPos  = 'LEADING' AND SELF.CurrencySymbol <> ''    !We need to skip the leading the currency symbol...
         xCurrencySymbol = CLIP(LEFT(SELF.CurrencySymbol))              !..It doesn't matter if there is space between symbol and number...
         xInitPos = xInitPos + LEN(xCurrencySymbol)
      END            
      !
      LOOP xPos = xLen TO xInitPos BY -1
         !
         !(2) Take care of trailing negative sign and/or trailing currency symbol....
         IF xPos = xLen 
            !Check for trailing negative sign...
            IF SELF.negativeSign = 'TRAILING'
               IF xStr[xPos] = '-'
                  xIsNegativeValue = TRUE
                  xPos -= 1
               END
            ELSIF SELF.negativeSign = 'BRACKETS'
               IF xStr[xPos] = ')'
                  xIsNegativeValue = TRUE
                  xPos -= 1
               END
            END
            !
            !Check for trailing currencysymbol....
            IF SELF.currencyPos  = 'TRAILING' AND SELF.CurrencySymbol <> ''        !We need to read the currency symbol...
               !CLIP the currencySymbol because we RIGHTed the string above....
               IF xStr[xPos - LEN(CLIP(SELF.CurrencySymbol)) + 1 : xPos] <> CLIP(SELF.CurrencySymbol)   !No currency symbol.... BAD PICTURE
                  xIsBadPicture = TRUE
                  DO ProcedureReturn
               ELSE
                  xPos = xPos - LEN(CLIP(SELF.CurrencySymbol))                     !Go to right before the currency symbol....
               END
            END
         END   
         !
         !(3)  Read the number...
         IF SELF.DecimalDigits > xDecimalsRead                  !Reading decimal part....
            IF __IsDigit(xStr[xPos])
               xDecimalPart   = xStr[xPos] & xDecimalPart       !..build the decimal portion...
               xDecimalsRead += 1
            ELSE
               IF SELF.strictDeformat                           !Strictly this if bad format, because
                  xIsBadPicture = TRUE                          !..decimals are INCOMPLETE.....    
                  DO ProcedureReturn
               ELSE                                             !Not strictly, just proceed to integer part...
                  xDecimalsRead = SELF.DecimalDigits            !..signal to go on...
               END
            END
         ELSE                                                   !We are done reading the decimal part....
            IF __IsDigit(xStr[xPos])
               xIntegerPart   = xStr[xPos] & xIntegerPart       !..build the integer portion...
               xIntegersRead += 1
            ELSE                                                !Check for leading negative sign....
               IF SELF.negativeSign = 'LEADING' AND xStr[xPos] = '-'
                  xIsNegativeValue = TRUE
               END
            END
         END
      END
      !
      !(3) BUILD the value
      xValue = CHOOSE(xIsNegativeValue = TRUE,'-','') & CHOOSE(xIntegerPart='','0',xIntegerPart) & '.' & xDecimalPart
      DO ProcedureReturn
   !::::::::::::::::::::::::::::::::::::::::::::::   
   !:::::SCIENTIFIC:::::::::::::::::::::::::::::::   
   OF 'E'
      xLen = LEN(CLIP(LEFT(pValue)))    !Trim "pValue"
      xStr &= NEW(CSTRING(xLen + 1))
      xStr = RIGHT(pValue,xLen)         !Right align value string....
      !
      !
      !(1) Rightmost 4 are the exponential portion with the sign...
      xExpS = RIGHT(pValue,4)
      xExp = xExpS[2 : 4]
      IF xExpS[1] = '-'
         xExp = xExp * -1
      END
      xLen = xLen - 4
      !
      !(2) Read the number....
      LOOP xPos = xLen TO xInitPos BY -1
         IF xDecimalsRead
            IF __IsDigit(xStr[xPos])
               xIntegerPart   = xStr[xPos] & xIntegerPart
            ELSIF xStr[xPos] = '-'
               xIsNegativeValue = TRUE
            END   
         ELSIF xStr[xPos] = SELF.decimalSeparator
            xDecimalsRead = TRUE
            CYCLE
         ELSE   
            IF __IsDigit(xStr[xPos])
               xDecimalPart   = xStr[xPos] & xDecimalPart       !..build the decimal portion...
            END
         END 
      END
      !
      !(3) BUILD the value
      xValue = CHOOSE(xIsNegativeValue = TRUE,'-','') & CHOOSE(xIntegerPart='','0',xIntegerPart) & '.' & xDecimalPart 
      xValue = xValue * 10^xExp
      DO ProcedureReturn     
   ELSE
      MESSAGE('Programming error.   TPictureClass.DEFORMAT currently only handles NUMERIC pictures, and this picture is: ' & SELF.Picture)
      DO ProcedureReturn
   END

ProcedureReturn ROUTINE
   IF NOT xStr &= NULL
      DISPOSE(xStr)
   END
   IF xIsBadPicture
      xValue = 0
   END
   RETURN xValue
   
   

!#######################################################################################################   
!#######################################################################################################   
!#######################################################################################################   
!#######################################################################################################   
!#######################################################################################################   
!#######################################################################################################   
!#######################################################################################################   
!#######################################################################################################   
!#######################################################################################################   
!#######################################################################################################   
!#######################################################################################################   
__IsDigit                         PROCEDURE(STRING pTestValue)
        CODE
    IF VAL(pTestValue) >= 48 AND VAL(pTestValue) <= 57
       RETURN TRUE
    ELSE
       RETURN FALSE
    END   

!
!Replace the "What" character (1 byte string) with the "With" character...
__replaceChar                     PROCEDURE(*CSTRING pStr,STRING pWhat,STRING pWith)
i               LONG
j               LONG
                CODE
         i = 1       
         LOOP 
            j = INSTRING(pWhat,pStr,1,i)
            IF j
               pStr[j] = pWith[1]
               i = j + 1
            ELSE
               BREAK
            END
         END
            
                
                